<?php

/**
 * @file
 * Contains \Drupal\editablevar\EditablevarImportForm.
 */

namespace Drupal\editablevar;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;

/**
 * Builds the form to import editablevars.
 */
class EditablevarImportForm extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'editablevar_export_form';
  }
  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['import_textarea'] = array(
      '#type' => 'textarea',
      '#title' => t('Editablevar Import'),
      '#required' => TRUE,
      '#default_value' => '',
    );
    $form['delete_before_import'] = array(
      '#type' => 'radios',
      '#title' => t('Delete existing groups before importing?'),
      '#options' => [
        '' => t('Do not delete and do not import anything. Please read the other options carefully and choose one.'),
        'delete_all' => t('Delete all existing groups before importing. Use this for mirroring the settings of a site.'),
        'delete_imported_groups' => t('Delete only those existing groups that will be imported. Use this if there are editablevar groups that will not be imported, but should be kept.'),
        'delete_nothing' => t('Do not delete existing groups and variables before importing. Use this if import should only create or overwrite variables, and leave other variables as they are.'),
      ],
      '#required' => TRUE,
      '#default_value' => '',
    );
    $form['actions'] = array(
      '#type' => 'actions',
      '#attributes' => array('class' => array('container-inline')),
    );
    $form['actions']['submit'] = array(
      '#type' => 'submit',
      '#value' => $this->t('Import'),
    );
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $import_textarea=$form_state->getValue('import_textarea');
    if (!preg_match('!"data_type.*editablevars"!', $import_textarea)) {
      $form_state->setErrorByName('import_textarea', $this->t('This data cannot be imported. Please copy-paste a valid JSON file in the textarea.'));
      return;
    }
    $data=json_decode($import_textarea, TRUE);
    if (!$data || !is_array($data) || empty($data['data_type']) || ($data['data_type']!='editablevars') || empty($data['data_format']) || ($data['data_format']!==1) || !isset($data['groups'])) {
      $form_state->setErrorByName('import_textarea', $this->t('This data cannot be imported. Please copy-paste a valid JSON file in the textarea.'));
      return;
    }
  }
  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $import_textarea=$form_state->getValue('import_textarea');
    $delete_before_import=$form_state->getValue('delete_before_import');
    $data=json_decode($import_textarea, TRUE);
    $group_count=0;
    $var_count=0;
    // Delete all existing groups and variables, if needed
    if ($delete_before_import=='delete_all') {
      foreach ($groups = EditablevarGroupStorage::load() as $group) {
        $group_count++;
        foreach ($variables=EditablevarVarStorage::load(array('group_record_id' => $group->record_id)) as $variable) {
          EditablevarVarStorage::delete((array)$variable);
          $var_count++;
        }
        EditablevarGroupStorage::delete((array)$group);
        \Drupal::messenger()->addStatus($this->t('The group %label and its variables have been deleted.', array('%label' => $group->name)));
      }
    }
    // Delete existing to-be-imported groups and variables, if needed
    elseif (($delete_before_import=='delete_imported_groups') && !empty($data['groups'])) {
      foreach ($data['groups'] as $group_data) {
        if (empty($group_data['id'])) {
          continue;
        }
        if ($group = EditablevarGroupStorage::loadById($group_data['id'])) {
          $group_count++;
          foreach ($variables=EditablevarVarStorage::load(array('group_record_id' => $group->record_id)) as $variable) {
            $var_count++;
            EditablevarVarStorage::delete((array)$variable);
          }
          EditablevarGroupStorage::delete((array)$group);
          \Drupal::messenger()->addStatus($this->t('The group %label and its variables have been deleted.', array('%label' => $group->name)));
        }
      }
    }
    if ($group_count) {
      \Drupal::messenger()->addStatus(t('Deleted @group_count groups and @var_count variables.', ['@group_count' => $group_count, '@var_count' => $var_count]));
    }
    $group_count=0;
    $var_count=0;
    if (!empty($data['groups'])) {
      foreach ($data['groups'] as $group_data) {
        $group_entry = array(
          'id' => $group_data['id'],
          'name' => $group_data['name'],
          'description' => $group_data['description'],
          'source_file' => $group_data['source_file'],
          'target_file' => $group_data['target_file'],
          'json_file' => $group_data['json_file'],
          'var_id_prefix' => $group_data['var_id_prefix'],
        );
        if ($group = EditablevarGroupStorage::loadById($group_data['id'])) {
          $group_entry['record_id']=$group->record_id;
          $return = EditablevarGroupStorage::update($group_entry);
        }
        else {
          $return = EditablevarGroupStorage::insert($group_entry);
        }
        if ($return && ($group = EditablevarGroupStorage::loadById($group_data['id']))) {
          $group_count++;
          if (!empty($group_data['variables'])) {
            foreach ($group_data['variables'] as $variable_data) {
              $var_entry = array(
                'id' => $variable_data['id'],
                'description' => $variable_data['description'],
                'value' => $variable_data['value'],
                'group_record_id' => $group->record_id,
              );
              if ($var = EditablevarVarStorage::loadById($variable_data['id'])) {
                $var_entry['record_id']=$var->record_id;
                $return = EditablevarVarStorage::update($var_entry);
              }
              else {
                $return = EditablevarVarStorage::insert($var_entry);
              }
              if ($return) {
                $var_count++;
              }
            }
          }
          \Drupal::messenger()->addStatus(t('Imported group @group', array('@group' => $group_data['name'])));
        }
      }
      \Drupal::messenger()->addStatus(t('Imported @group_count groups and @var_count variables.', ['@group_count' => $group_count, '@var_count' => $var_count]));
    }
    $form_state->setRedirectUrl(new Url('editablevar.group_list'));
  }
}
